package com.yutu.base.service;

import com.alibaba.fastjson.JSONObject;
import com.yutu.base.entity.Response;
import com.yutu.base.utils.HttpsSendData;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingDeque;

/**
 * 销售明细报表相关操作
 */
@Service
public class SaleDetailService implements CommandLineRunner {

    @Value("${spring.urlConfig.overseasAirTicketUrl}")
    private String overseasAirTicketUrl;

    private static Logger logger = Logger.getLogger("saleDetail");

    //任务消息队列，元素 调用AirTicket工程刷新SaleDetail表数据 的参数Map<String,String>
    public static BlockingQueue<Map<String,String>> updateSaleDetailQueue = new LinkedBlockingDeque<>();
    //提供入队列方法
    public static void setUpdateSaleDetailQueue(Map<String,String> paramsMap) {
        try{
            updateSaleDetailQueue.put(paramsMap);
        }catch (Exception e){
            logger.error("updateSaleDetailQueue 队列,入队列异常,参数:" + paramsMap);
        }
    }

    /**
     * 采购信息更新时，更新报表数据，以队列方式执行
     * @param requestBody
     * @return
     */
    public Response updateSaleDetail(String requestBody){
        System.out.println(requestBody);
        if(StringUtils.isBlank(requestBody)){
            return Response.error("[updateSaleDetail] 参数不合法,requestBody=null");
        }
        JSONObject reqObj = JSONObject.parseObject(requestBody);
        String customerId = reqObj.getString("customerId");
        String startTime = reqObj.getString("startTime");
        String endTime = reqObj.getString("endTime");
        if(StringUtils.isBlank(customerId) || StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime)){
            return Response.error("[updateSaleDetail] 参数不合法");
        }
        try{
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Date endDate = sdf.parse(endTime);
            if(endDate.getTime() > new Date().getTime()){
                return Response.error("[updateSaleDetail] 当天订单采购信息修改不触发刷新报表数据，endTime="+endTime);
            }
            Map<String,String> paramsMap = new HashMap<>();
            paramsMap.put("customerId",customerId);
            paramsMap.put("startTime",startTime);
            paramsMap.put("endTime",endTime);
            setUpdateSaleDetailQueue(paramsMap);//入队列
            logger.info("[updateSaleDetail] 采购信息变动，刷新报表数据任务已加入队列，参数："+paramsMap);
            logger.info("[updateSaleDetail] 当前队列长度 updateSaleDetailQueue.size()="+updateSaleDetailQueue.size());
            return Response.success("[updateSaleDetail] 已加入队列");
        }catch (Exception e){
            logger.error("updateSaleDetail 发生异常",e);
        }
        return Response.error("未知原因失败");
    }

    @Override
    public void run(String... args) throws Exception {
        consumeUpdateSaleDetailQueue();
    }

    //消费响应消息队列
    private void consumeUpdateSaleDetailQueue(){
        ExecutorService threadPool = Executors.newFixedThreadPool(1);
        Runnable consumeTask = new Runnable() {
            @Override
            public void run() {
                while (true){
                    logger.info("[updateSaleDetailQueue] 刷新报表队列 当前长度："+updateSaleDetailQueue.size());
                    Map<String,String> map = null;
                    try {
                        //调用AirTicket工程刷新SaleDetail表数据
                        map = updateSaleDetailQueue.take();
                        HttpsSendData.send(overseasAirTicketUrl+"autoSaleDetai/updateSaleDetail.do",map);
                        logger.info("刷新报表已执行，参数："+map);
                    }catch (Exception e) {
                        logger.error("刷新报表队列执行任务时异常,参数"+map);
                    }
                }
            }
        };
        threadPool.execute(consumeTask);
    }
}
